@@ -18,6 +18,7 @@ import base64
 import contextlib
 import json
 import time
+import datetime
 
 import requests
 import aiohttp
@@ -35,20 +36,37 @@ import octobot_commons.authentication as authentication
 import octobot_commons.configuration as commons_configuration
 
 
+def _selected_bot_update(func):
+    async def wrapper(*args, **kwargs):
+        self = args[0]
+        await self.gql_login_if_required()
+        if self._update_bot_lock is None:
+            self._update_bot_lock = asyncio.Lock()
+        async with self._update_bot_lock:
+            self.logger.debug(f"@selected_bot_update: entering {func.__name__}")
+            updated_bot = await func(*args, **kwargs)
+        self.logger.debug(f"@selected_bot_update: exited {func.__name__}")
+        self.user_account.set_selected_bot_raw_data(updated_bot)
+        return updated_bot
+
+    return wrapper
+
+
 class CommunityAuthentication(authentication.Authenticator):
     """
     Authentication utility
     """
     ALLOWED_TIME_DELAY = 1 * commons_constants.MINUTE_TO_SECONDS
+    NEW_ACCOUNT_INITIALIZE_TIMEOUT = 1 * commons_constants.MINUTE_TO_SECONDS
     LOGIN_TIMEOUT = 20
     BOT_NOT_FOUND_RETRY_DELAY = 1
     AUTHORIZATION_HEADER = "authorization"
     SESSION_HEADER = "X-Session"
     GQL_AUTHORIZATION_HEADER = "Authorization"
 
-    def __init__(self, authentication_url, feed_url, config=None):
+    def __init__(self, feed_url, config=None):
         super().__init__()
-        self.authentication_url = authentication_url
+        self.authentication_url = identifiers_provider.IdentifiersProvider.BACKEND_AUTH_URL
         self.feed_url = feed_url
         self.edited_config = config
         self.initialized_event = None
@@ -56,20 +74,20 @@ class CommunityAuthentication(authentication.Authenticator):
         self.user_account = community_user_account.CommunityUserAccount()
         self._community_token = self._get_encoded_community_token()
         self._auth_token = None
-        self._session = requests.Session()
-        self._aiohttp_session = None
-        self._cache = {}
+        self._backend_session = requests.Session()
+        self._aiohttp_gql_session = None
+        self._aiohttp_backend_session = None
         self._fetch_account_task = None
         self._restart_task = None
         self._community_feed = None
         self._login_completed = None
+        self._update_bot_lock = None
 
         self._update_sessions_headers()
 
     @staticmethod
     def create(configuration: commons_configuration.Configuration):
         return CommunityAuthentication.instance(
-            identifiers_provider.IdentifiersProvider.BACKEND_AUTH_URL,
             identifiers_provider.IdentifiersProvider.FEED_URL,
             config=configuration,
         )
@@ -98,6 +116,25 @@ class CommunityAuthentication(authentication.Authenticator):
             return None
         return self._community_feed.last_message_time
 
+    def has_filled_form(self, form_id):
+        if not self.user_account.has_user_data():
+            raise authentication.AuthenticationRequired()
+        return form_id in self.user_account.get_filled_forms_ids()
+
+    async def register_filled_form(self, form_id):
+        if self.has_filled_form(form_id):
+            return
+        updated_filled_forms = self.user_account.get_filled_forms_ids()
+        updated_filled_forms.append(form_id)
+        await self._update_account_metadata({
+            self.user_account.FILLED_FORMS: updated_filled_forms
+        })
+
+    def get_user_id(self):
+        if not self.user_account.has_user_data():
+            raise authentication.AuthenticationRequired()
+        return self.user_account.get_user_id()
+
     def get_deployment_url(self):
         return self.user_account.get_bot_deployment_url()
 
@@ -118,13 +155,15 @@ class CommunityAuthentication(authentication.Authenticator):
         self._update_supports(200, self._supports_mock())
         return
         # TODO use real support fetch when implemented
-        async with self._aiohttp_session.get("supports_url") as resp:
+        async with self._aiohttp_gql_session.get("supports_url") as resp:
             self._update_supports(resp.status, await resp.json())
 
     def ensure_async_loop(self):
         # elements should be bound to the current loop
-        if self._aiohttp_session is not None and self._aiohttp_session.loop is not asyncio.get_event_loop():
-            self._aiohttp_session = None
+        if self._aiohttp_gql_session is not None and self._aiohttp_gql_session.loop is not asyncio.get_event_loop():
+            self._aiohttp_gql_session = None
+        if self._aiohttp_backend_session is not None and self._aiohttp_backend_session.loop is not asyncio.get_event_loop():
+            self._aiohttp_backend_session = None
         if self.initialized_event is not None and self.initialized_event._loop is not asyncio.get_event_loop():
             should_set = self.initialized_event.is_set()
             self.initialized_event = asyncio.Event()
@@ -230,17 +269,15 @@ class CommunityAuthentication(authentication.Authenticator):
     @contextlib.asynccontextmanager
     async def _authenticated_qgl_session(self):
         await self.gql_login_if_required()
-        session = self.get_aiohttp_session()
+        session = self.get_gql_aiohttp_session()
         try:
-            session.headers[self.GQL_AUTHORIZATION_HEADER] = f"Bearer {self.user_account.gql_access_token}"
             yield session
         except authentication.AuthenticationRequired:
             # reset token to force re-login
             self.user_account.gql_access_token = None
+            if CommunityAuthentication.GQL_AUTHORIZATION_HEADER in session.headers:
+                session.headers.pop(CommunityAuthentication.GQL_AUTHORIZATION_HEADER)
             raise
-        finally:
-            if self.GQL_AUTHORIZATION_HEADER in session.headers:
-                session.headers.pop(self.GQL_AUTHORIZATION_HEADER)
 
     async def wait_for_login_if_processing(self):
         if self._login_completed is not None:
@@ -253,7 +290,7 @@ class CommunityAuthentication(authentication.Authenticator):
             return
         try:
             token = self.user_account.get_graph_token()
-        except KeyError as e:
+        except (KeyError, TypeError) as e:
             raise authentication.AuthenticationRequired("Authentication required") from e
         async with self.get_aiohttp_session().post(
                 identifiers_provider.IdentifiersProvider.GQL_AUTH_URL, json={"key": token}
@@ -262,12 +299,13 @@ class CommunityAuthentication(authentication.Authenticator):
             if resp.status == 200:
                 self.user_account.gql_access_token = json_resp["access_token"]
                 self.user_account.gql_user_id = json_resp["user_id"]
+                self._update_sessions_headers()
             else:
                 raise authentication.FailedAuthentication(f"Failed to authenticate to graphql server: "
                                                           f"status: {resp.status}, data: {json_resp}")
 
     def can_authenticate(self):
-        return "todo" not in self.authentication_url
+        return "todo" not in self.authentication_url    # pylint: disable=E1135
 
     def must_be_authenticated_through_authenticator(self):
         return constants.IS_CLOUD_ENV
@@ -283,7 +321,7 @@ class CommunityAuthentication(authentication.Authenticator):
             params["password_token"] = password_token
         else:
             params["password"] = password
-        resp = self._session.post(self.authentication_url, json=params)
+        resp = self._backend_session.post(self.authentication_url, json=params)
         try:
             self._handle_auth_result(resp.status_code, resp.json(), resp.headers)
         except json.JSONDecodeError as e:
@@ -294,6 +332,63 @@ class CommunityAuthentication(authentication.Authenticator):
             await self._on_authenticated()
             self.initialized_event.set()
 
+    async def register(self, email, password):
+        if self.must_be_authenticated_through_authenticator():
+            raise authentication.AuthenticationError("Creating a new account is not authorized on this environment.")
+        # always logout before creating a new account
+        self.logout()
+        self._ensure_community_url()
+        params = {
+            "email": email,
+            "password": password,
+            community_user_account.CommunityUserAccount.METADATA: {
+                "hasRegisteredFromSelfHosted": True,
+            }
+        }
+        async with self.get_aiohttp_session().post(
+                identifiers_provider.IdentifiersProvider.BACKEND_ACCOUNT_URL, json=params
+        ) as resp:
+            try:
+                self._handle_register_result(resp.status, await resp.json(), resp.headers)
+            except json.JSONDecodeError as e:
+                raise authentication.FailedAuthentication(e)
+        if self.is_logged_in():
+            await self._on_register()
+
+    async def _update_account_metadata(self, metadata_update):
+        params = {
+            community_user_account.CommunityUserAccount.METADATA: metadata_update
+        }
+        async with self.get_aiohttp_session().put(
+                identifiers_provider.IdentifiersProvider.BACKEND_ACCOUNT_URL, json=params
+        ) as resp:
+            self._handle_account_update_result(resp.status, await resp.json(), resp.headers)
+
+    async def _on_register(self):
+        if self.initialized_event is None:
+            self.initialized_event = asyncio.Event()
+        t0 = time.time()
+        try:
+            while not self._is_new_account_fully_initialized() \
+                    and time.time() - t0 < self.NEW_ACCOUNT_INITIALIZE_TIMEOUT:
+                await self._async_check_auth()
+        finally:
+            if self._is_new_account_fully_initialized():
+                self.logger.debug(f"New account has been fully initialized, now processing authenticated checkups")
+                await self._on_authenticated()
+                self.logger.debug(f"Authenticated checkups complete")
+            else:
+                self.logger.error(f"New account has not been fully initialized")
+            self.initialized_event.set()
+
+    def _is_new_account_fully_initialized(self):
+        # required elements to be fully initialized
+        try:
+            self.user_account.get_graph_token()
+        except (KeyError, TypeError):
+            return False
+        return True
+
     async def update_selected_bot(self):
         self.user_account.flush_bot_details()
         await self._load_bot_if_selected()
@@ -357,6 +452,67 @@ class CommunityAuthentication(authentication.Authenticator):
             for profile_data in subscribed_profiles["data"]
         ]
 
+    async def update_trades(self, trades: list):
+        """
+        Updates authenticated account trades
+        """
+        if not self.is_logged_in():
+            return
+        try:
+            formatted_trades = [
+                {
+                    "date": self._get_graphql_formatted_time(trade.executed_time),
+                    "exchange": trade.exchange_manager.exchange_name,
+                    "price": str(trade.executed_price),
+                    "quantity": str(trade.executed_quantity),
+                    "symbol": trade.symbol,
+                    "type": trade.trade_type.value,
+                }
+                for trade in trades
+            ]
+            await self._update_bot_trades(formatted_trades)
+        except Exception as err:
+            self.logger.exception(err, True, f"Error when updating community trades {err}")
+
+    async def update_portfolio(self, current_value: dict, initial_value: dict,
+                               unit: str, content: dict, history: dict, price_by_asset: dict):
+        """
+        Updates authenticated account portfolio
+        """
+        if not self.is_logged_in():
+            return
+        try:
+            ref_market_current_value = current_value[unit]
+            ref_market_initial_value = initial_value[unit]
+            formatted_content = [
+                {
+                    "asset": key,
+                    "quantity": str(quantity[commons_constants.PORTFOLIO_TOTAL]),
+                    "value": str(quantity[commons_constants.PORTFOLIO_TOTAL] * float(price_by_asset.get(key, 0))),
+                }
+                for key, quantity in content.items()
+            ]
+            formatted_history = []
+            try:
+                formatted_history = [
+                    {
+                        "date": self._get_graphql_formatted_time(timestamp),
+                        "value": str(value[unit])
+                    }
+                    for timestamp, value in history.items()
+                    if unit in value
+                ]
+            except KeyError:
+                pass
+            if not formatted_history:
+                return
+            await self._update_bot_portfolio(
+                ref_market_current_value, ref_market_initial_value, unit,
+                formatted_content, formatted_history
+            )
+        except Exception as err:
+            self.logger.exception(err, True, f"Error when updating community portfolio {err}")
+
     def _get_self_hosted_bots(self, bots):
         return [
             bot
@@ -368,29 +524,23 @@ class CommunityAuthentication(authentication.Authenticator):
         await self._update_feed_device_uuid_and_restart_feed_if_necessary()
 
     async def _fetch_startup_info(self, bot_id):
-        query, variables = graphql_requests.select_startup_info_query(bot_id)
-        return await self.async_graphql_query(query, "getBotStartupInfo", variables=variables, expected_code=200)
+        return await self._execute_request(graphql_requests.select_startup_info_query, bot_id)
 
     async def _fetch_subscribed_profiles(self):
-        query, variables = graphql_requests.select_subscribed_profiles_query()
-        return await self.async_graphql_query(query, "getSubscribedProfiles", variables=variables, expected_code=200)
+        return await self._execute_request(graphql_requests.select_subscribed_profiles_query)
 
     async def _fetch_bots(self):
-        query, variables = graphql_requests.select_bots_query()
-        return await self.async_graphql_query(query, "bots", variables=variables, expected_code=200)
+        return await self._execute_request(graphql_requests.select_bots_query)
 
     async def _fetch_bot(self, bot_id):
-        query, variables = graphql_requests.select_bot_query(bot_id)
-        return await self.async_graphql_query(query, "bot", variables=variables, expected_code=200)
+        return await self._execute_request(graphql_requests.select_bot_query, bot_id)
 
     async def create_new_bot(self):
         await self.gql_login_if_required()
-        query, variables = graphql_requests.create_bot_query(not constants.IS_CLOUD_ENV)
-        return await self.async_graphql_query(query, "createBot", variables=variables, expected_code=200)
+        return await self._execute_request(graphql_requests.create_bot_query, not constants.IS_CLOUD_ENV)
 
     async def _create_new_bot_device(self, bot_id):
-        query, variables = graphql_requests.create_bot_device_query(bot_id)
-        await self.async_graphql_query(query, "createBotDevice", variables=variables, expected_code=200)
+        await self._execute_request(graphql_requests.create_bot_device_query, bot_id)
         # issue with createBotDevice not always returning the created device, fetch bot again to fetch device with it
         return await self._fetch_bot(bot_id)
 
@@ -408,15 +558,34 @@ class CommunityAuthentication(authentication.Authenticator):
             await self._ensure_bot_device()
             self._restart_task = asyncio.create_task(self._community_feed.restart())
 
+    @_selected_bot_update
     async def update_bot_config_and_stats(self, profile_name, profitability):
-        await self.gql_login_if_required()
-        query, variables = graphql_requests.update_bot_config_and_stats_query(
+        return await self._execute_request(
+            graphql_requests.update_bot_config_and_stats_query,
             self.user_account.gql_bot_id,
             profile_name,
             profitability
         )
-        updated_bot = await self.async_graphql_query(query, "updateOneBot", variables=variables, expected_code=200)
-        self.user_account.set_selected_bot_raw_data(updated_bot)
+
+    @_selected_bot_update
+    async def _update_bot_trades(self, trades):
+        return await self._execute_request(
+            graphql_requests.update_bot_trades_query,
+            self.user_account.gql_bot_id,
+            trades
+        )
+
+    @_selected_bot_update
+    async def _update_bot_portfolio(self, current_value, initial_value, unit, content, history):
+        return await self._execute_request(
+            graphql_requests.update_bot_portfolio_query,
+            self.user_account.gql_bot_id,
+            current_value, initial_value, unit, content, history
+        )
+
+    async def _execute_request(self, request_factory, *args, **kwargs):
+        query, variables, query_name = request_factory(*args, **kwargs)
+        return await self.async_graphql_query(query, query_name, variables=variables, expected_code=200)
 
     def logout(self):
         """
@@ -424,7 +593,6 @@ class CommunityAuthentication(authentication.Authenticator):
         Warning: also call stop_feeds if feeds have to be stopped (not done here to keep method sync)
         """
         self._reset_tokens()
-        self.clear_cache()
         self.remove_login_detail()
         for task in (self._restart_task, self._fetch_account_task):
             if task is not None and not task.done():
@@ -437,34 +605,12 @@ class CommunityAuthentication(authentication.Authenticator):
         if self._community_feed is not None and self._community_feed.is_connected():
             await self._community_feed.stop()
 
-    def clear_cache(self):
-        self._cache = {}
-
-    @authentication.authenticated
-    def get(self, url, params=None, allow_cache=False, **kwargs):
-        if allow_cache:
-            if url not in self._cache:
-                self._cache[url] = self._session.get(url, params=params, **kwargs)
-            return self._cache[url]
-        else:
-            return self._session.get(url, params=params, **kwargs)
-
-    @authentication.authenticated
-    def download(self, url, target_file, params=None, **kwargs):
-        with self._session.get(url, stream=True, params=params, **kwargs) as r:
-            r.raise_for_status()
-            with open(target_file, 'wb') as f:
-                for chunk in r.iter_content(chunk_size=8192):
-                    f.write(chunk)
-        return target_file
-
-    @authentication.authenticated
-    def post(self, url, data=None, json=None, **kwargs):
-        return self._session.post(url, data=data, json=json, **kwargs)
-
     def is_logged_in(self):
         return bool(self._auth_token and self.user_account.has_user_data())
 
+    def has_login_info(self):
+        return bool(self._get_saved_token())
+
     def ensure_token_validity(self):
         if not self.is_logged_in():
             # try to login with saved token if available
@@ -480,10 +626,16 @@ class CommunityAuthentication(authentication.Authenticator):
         self.logger.debug("Removed community login data")
 
     def get_aiohttp_session(self):
-        if self._aiohttp_session is None:
-            self._aiohttp_session = aiohttp.ClientSession()
+        if self._aiohttp_backend_session is None:
+            self._aiohttp_backend_session = aiohttp.ClientSession()
             self._update_sessions_headers()
-        return self._aiohttp_session
+        return self._aiohttp_backend_session
+
+    def get_gql_aiohttp_session(self):
+        if self._aiohttp_gql_session is None:
+            self._aiohttp_gql_session = aiohttp.ClientSession()
+            self._update_sessions_headers()
+        return self._aiohttp_gql_session
 
     async def stop(self):
         self.logger.debug("Stopping ...")
@@ -492,8 +644,10 @@ class CommunityAuthentication(authentication.Authenticator):
             self._fetch_account_task.cancel()
         if self._restart_task is not None and not self._restart_task.done():
             self._restart_task.cancel()
-        if self._aiohttp_session is not None:
-            await self._aiohttp_session.close()
+        if self._aiohttp_backend_session is not None:
+            await self._aiohttp_backend_session.close()
+        if self._aiohttp_gql_session is not None:
+            await self._aiohttp_gql_session.close()
         self.logger.debug("Stopped")
 
     def _update_supports(self, resp_status, json_data):
@@ -608,7 +762,7 @@ class CommunityAuthentication(authentication.Authenticator):
 
     def _check_auth(self):
         with self._auth_context():
-            with self._session.get(f"{identifiers_provider.IdentifiersProvider.BACKEND_API_URL}/account") as resp:
+            with self._backend_session.get(identifiers_provider.IdentifiersProvider.BACKEND_ACCOUNT_URL) as resp:
                 self._handle_auth_result(resp.status_code, resp.json(), resp.headers)
 
     async def _async_check_auth(self):
@@ -617,7 +771,7 @@ class CommunityAuthentication(authentication.Authenticator):
         self._login_completed.clear()
         with self._auth_context():
             async with self.get_aiohttp_session().get(
-                    f"{identifiers_provider.IdentifiersProvider.BACKEND_API_URL}/account"
+                    identifiers_provider.IdentifiersProvider.BACKEND_ACCOUNT_URL
             ) as resp:
                 try:
                     self._handle_auth_result(resp.status, await resp.json(), resp.headers)
@@ -653,13 +807,47 @@ class CommunityAuthentication(authentication.Authenticator):
                                                               f"re-login to your community account")
         raise authentication.AuthenticationError(f"Error code: {status_code}")
 
-    def _update_sessions_headers(self):
-        headers = self.get_headers()
-        self._session.headers.update(headers)
-        if self._aiohttp_session is not None:
-            self._aiohttp_session.headers.update(headers)
+    def _handle_register_result(self, status_code, json_resp, reps_headers):
+        if status_code == 200 and json_resp is not None:
+            if "id" in json_resp:
+                # successfully created account
+                if new_token := reps_headers.get(self.SESSION_HEADER):
+                    self._auth_token = new_token
+                    self._save_login_token(self._auth_token)
+                    self._update_sessions_headers()
+                self.user_account.set_profile_raw_data(json_resp)
+                return
+            # error on create account
+            error_messages = [
+                f"{field}: {details['message']}"
+                for field, details in json_resp.items()
+            ]
+            raise authentication.AuthenticationError(", ".join(error_messages))
+        raise authentication.AuthenticationError(f"Unexpected error when creating account: code: {status_code} ({json_resp})")
 
-    def get_headers(self):
+    def _handle_account_update_result(self, status_code, json_resp, reps_headers):
+        if status_code == 200 and json_resp is not None:
+            if "id" in json_resp:
+                # successfully updated account
+                self.user_account.set_profile_raw_data(json_resp)
+                return
+            # error on update account
+            error_messages = [
+                f"{field}: {details['message']}"
+                for field, details in json_resp.items()
+            ]
+            raise authentication.AccountUpdateError(", ".join(error_messages))
+        raise authentication.AccountUpdateError(f"Unexpected error when updating account: code: {status_code} ({json_resp})")
+
+    def _update_sessions_headers(self):
+        backend_headers = self.get_backend_headers()
+        self._backend_session.headers.update(backend_headers)
+        if self._aiohttp_backend_session is not None:
+            self._aiohttp_backend_session.headers.update(backend_headers)
+        if self._aiohttp_gql_session is not None:
+            self._aiohttp_gql_session.headers.update(self.get_gql_headers())
+
+    def get_backend_headers(self):
         headers = {
             CommunityAuthentication.AUTHORIZATION_HEADER: f"Basic {self._community_token}",
         }
@@ -667,13 +855,25 @@ class CommunityAuthentication(authentication.Authenticator):
             headers[CommunityAuthentication.SESSION_HEADER] = self._auth_token
         return headers
 
+    def get_gql_headers(self):
+        headers = {}
+        if self.user_account.gql_access_token is not None:
+            headers[CommunityAuthentication.GQL_AUTHORIZATION_HEADER] = f"Bearer {self.user_account.gql_access_token}"
+        return headers
+
     def _reset_tokens(self):
         self._auth_token = None
         self.user_account.flush()
-        self._session.headers.pop(CommunityAuthentication.SESSION_HEADER, None)
-        if self._aiohttp_session is not None:
-            self._aiohttp_session.headers.pop(CommunityAuthentication.SESSION_HEADER, None)
+        self._backend_session.headers.pop(CommunityAuthentication.SESSION_HEADER, None)
+        if self._aiohttp_backend_session is not None:
+            self._aiohttp_backend_session.headers.pop(CommunityAuthentication.SESSION_HEADER, None)
+        if self._aiohttp_gql_session is not None:
+            self._aiohttp_gql_session.headers.pop(CommunityAuthentication.GQL_AUTHORIZATION_HEADER, None)
 
     @staticmethod
     def _get_encoded_community_token():
         return base64.encodebytes(identifiers_provider.IdentifiersProvider.BACKEND_PUBLIC_TOKEN.encode()).decode().strip()
+
+    @staticmethod
+    def _get_graphql_formatted_time(timestamp):
+        return f"{datetime.datetime.utcfromtimestamp(timestamp).isoformat('T')}Z"
