@@ -24,6 +24,8 @@ class CommunityUserAccount:
     BOT_DEPLOYMENT = "deployment"
     BOT_DEPLOYMENT_TYPE = "type"
     SELF_HOSTED_BOT_DEPLOYMENT_TYPE = "self-hosted"
+    METADATA = "metadata"
+    FILLED_FORMS = "filledForms"
     NO_SELECTED_BOT_DESC = "No selected bot. Please select a bot to enable your community features."
 
     def __init__(self):
@@ -45,12 +47,18 @@ class CommunityUserAccount:
     def get_email(self):
         return self._profile_raw_data["email"]
 
+    def get_user_id(self):
+        return self._profile_raw_data["id"]
+
     def get_graph_token(self):
         return self._get_user_data_content()["graph_token"]
 
     def get_has_donated(self):
         return self._get_user_data_content()["has_donated"]
 
+    def get_filled_forms_ids(self):
+        return self._get_user_data_metadata().get(self.FILLED_FORMS, [])
+
     def get_all_user_bots_raw_data(self):
         return self._all_user_bots_raw_data
 
@@ -113,6 +121,9 @@ class CommunityUserAccount:
     def _get_user_data_content(self):
         return self._profile_raw_data[self.USER_DATA_CONTENT]
 
+    def _get_user_data_metadata(self):
+        return self._profile_raw_data.get(self.METADATA, {})
+
     def _ensure_selected_bot_data(self):
         if self._selected_bot_raw_data is None:
             raise errors.BotError(self.NO_SELECTED_BOT_DESC)
